{% extends "base.html" %}

{% block title %}
	DNOs &raquo; {{dno.dno_code}} &raquo; LLFCs &raquo; {{llfc.code}} &raquo; LAFs
{% endblock %}

{% block nav %}
	<a href="/e/dnos">DNOs</a> &raquo;
	<a href="/e/dnos/{{dno.id}}">{{dno.dno_code}}</a> &raquo;
	<a href="/e/llfcs?dno_id={{dno.id}}">LLFCs</a> &raquo;
  <a href="/e/llfcs/{{llfc.id}}">{{llfc.code}}</a> &raquo;
	LAFs
{% endblock %}

{% block content %}
	<form action="/e/lafs">
		<fieldset>
			<legend>Show data</legend>	
			<input type="hidden" name="llfc_id" value="{{llfc.id}}">
			<label>Month</label> {{input_date(resolution='month')}}
			<input type="submit" value="Show">
		</fieldset>
	</form>

	<table class="sticky">
			<caption>LAFs</caption>
		<thead>
			<tr>
				<th>Timestamp</th>
				<th>Value</th>
			</tr>
		</thead>
		<tbody>
			{% for laf in lafs %}
				<tr>
					<td style="white-space: nowrap">{{laf.timestamp|hh_format}}</td>
					<td>{{laf.value}}</td>
				</tr>
			{% endfor %}
		</tbody>
	</table>

{% endblock %}
