{% extends "base.html" %}

{% block title %}
	&raquo; MOP Contracts &raquo; {{batch_file.batch.contract.name}} &raquo;
	Batches &raquo; {{batch_file.batch.reference}} &raquo; {{batch_file.name}}
{% endblock %}

{% block nav %}
	<a href="/e/mop_contracts">MOP Contracts</a> &raquo; 
	<a href="/e/mop_contracts/{{batch_file.batch.contract.id}}"
		>{{batch_file.batch.contract.name}}</a> &raquo; 
	<a href="/e/mop_contracts/{{batch_file.batch.contract.id}}/batches">Batches</a> &raquo; 
	<a href="/e/mop_batches/{{batch_file.batch.id}}">{{batch_file.batch.reference}}</a>
 	&raquo; 
	{{batch_file.filename}} [<a href="/e/mop_batch_files/{{batch_file.id}}/edit">edit</a>]
{% endblock %}

{% block content %}

	<table>
		<caption>Billing File</caption>	
		<thead>
			<tr>
				<th>Filename</th>
				<th>Upload Timestamp</th>
				<th>Parser</th>
				<th>Download</th>
			</tr>
		</thead>
		<tbody>
			<tr>
				<td>{{batch_file.filename}}</td>
				<td>{{batch_file.upload_timestamp|hh_format}}</td>
				<td>{{batch_file.parser_name}}</td>
				<td><a href="/e/mop_batch_files/{{batch_file.id}}/download">Download</a></td>
			</tr>
		</tbody>
	</table>

{% endblock %}
