{% extends "base.html" %}

{% block title %}
	&raquo; DNOs &raquo; {{llfc.dno.dno_code}} &raquo; LLFCs &raquo;
 	{{llfc.code}}
{% endblock %}

{% block nav %}
	<a href="/e/dnos">DNOs</a> &raquo; 
	<a href="/e/dnos/{{llfc.dno.id}}">{{llfc.dno.dno_code}}</a> &raquo; 
	<a href="/e/llfcs?dno_id={{llfc.dno.id}}">LLFCs</a> &raquo; {{llfc.code}}
	[<a href="/e/llfcs/{{llfc.id}}/edit">edit</a>]
{% endblock %}

{% block content %}
	<table>
		<tr>
			<th>Code</th>
			<td>{{llfc.code}}</td>
		</tr>
		<tr>
			<th>Description</th>
			<td>{{llfc.description}}</td>
		</tr>
		<tr>
			<th>Voltage Level</th>
			<td>{{llfc.voltage_level.code}} - {{llfc.voltage_level.name}}</td>
		</tr>
		<tr>
			<th>Is Substation?</th>
			<td>{{llfc.is_substation}}</td>
		</tr>
		<tr>
			<th>Is Import?</th>
			<td>{{llfc.is_import}}</td>
		</tr>
		<tr>
			<th>Valid From</th>
			<td>{{llfc.valid_from|hh_format}}</td>
		</tr>
		<tr>
			<th>Valid To</th>
			<td>{{llfc.valid_to|hh_format}}</td>
		</tr>
		<tr>
			<th>Loss Adjustment Factors</th>
			<td>
				<a href="/e/lafs?llfc_id={{llfc.id}}&amp;timestamp_year={{now.year}}&amp;timestamp_month={{now.month}}">LAFs</a>
			</td>
		</tr>
	</table>

{% endblock %}
