{% extends "base.html" %}

{% block title %}
	&raquo; Site Snags &raquo; Edit
{% endblock %}

{% block nav %}
	<a href="/e/site_snags">Site Snags</a> &raquo; Edit
{% endblock %}

{% block content %}
	<form method="post">
		<fieldset>
			<legend>Bulk ignore</legend>	
			<label>Ignore all snags before</label> {{input_date('ignore')}}
				<input type="submit" name="ignore" value="Ignore">
		</fieldset>
	</form>
{% endblock %}
