{% extends "base.html" %}

{% block title %}
	&raquo; Sites &raquo; {{site.code}} {{site.name}} &raquo; Add Electricity SUpply
{% endblock %}

{% block nav %}
	<a href="/sites">Sites</a> &raquo;
	<a href="/sites/{{site.id}}">{{site.code}} {{site.name}}</a> &raquo;
 	Add Electricity Supply
{% endblock %}

{% block content %}
	<form action="/e/sites/{{site.id}}/add_e_supply" method="post">
		<fieldset hx-get="/e/sites/{{site.id}}/add_e_supply/form" hx-params="*"
			 hx-trigger="load, change" hx-include="this">
			{{ input_hidden('source_id')}}
			<input type="hidden" name="cop_id" value="{{request.values.cop_id}}">
			<input type="hidden" name="comm_id" value="{{request.values.comm_id}}">
			<input type="hidden" name="energisation_status_id"
					value="{{request.values.energisation_status_id}}">
			<input type="hidden" name="pc_id" value="{{request.values.pc_id}}">
			<input type="hidden" name="dtc_meter_type_id" value="{{request.values.dtc_meter_type_id}}">
			<input type="hidden" name="dno_id" value="{{request.values.dno_id}}">
			<input type="hidden" name="gsp_group_id" value="{{request.values.gsp_group_id}}">
			<input type="hidden" name="mtc_participant_id"
		 			value="{{request.values.mtc_participant_id}}">
			<input type="hidden" name="name" value="{{request.values.name}}">
			<input type="hidden" name="mop_contract_id"
		 			value="{{request.values.mop_contract_id}}">
			<input type="hidden" name="mop_account" value="{{request.values.mop_account}}">
			<input type="hidden" name="dc_contract_id"
		 			value="{{request.values.dc_contract_id}}">
			<input type="hidden" name="dc_account" value="{{request.values.dc_account}}">
			<input type="hidden" name="has_imp_mpan" value="{{request.values.has_imp_mpan}}">
			<input type="hidden" name="imp_mpan_core"
		 			value="{{request.values.imp_mpan_core}}">
			<input type="hidden" name="imp_supplier_account"
					value="{{request.values.imp_supplier_account}}">
			<input type="hidden" name="imp_llfc_id"
					value="{{request.values.imp_llfc_id}}">
			<input type="hidden" name="has_exp_mpan" value="{{request.values.has_exp_mpan}}">
			<input type="hidden" name="exp_mpan_core"
		 			value="{{request.values.exp_mpan_core}}">
			<input type="hidden" name="exp_supplier_account"
					value="{{request.values.exp_supplier_account}}">
			<input type="hidden" name="exp_llfc_id"
					value="{{request.values.exp_llfc_id}}">
			<input type="hidden" name="ssc_id" value="{{request.values.ssc_id}}">
			<input type="hidden" name="imp_supplier_contract_id" value="{{request.values.imp_supplier_contract_id}}">
			<input type="hidden" name="exp_supplier_contract_id" value="{{request.values.exp_supplier_contract_id}}">
		</fieldset>
	</form>
{% endblock %}
